const fs = require('fs');
const canvas = require('canvas');
const XMLHttpRequest = require("xmlhttprequest").XMLHttpRequest;


//if .wasm file placed in not current folder
//var DEF_WASM_PATH = "/home/John/Desktop/nodejs/datasymbol-sdk.wasm";

var Module = {
	decHandle: 0,

	locateFile: function (fName) {
		if ((fName == 'datasymbol-sdk.wasm') && (typeof DEF_WASM_PATH !== 'undefined'))
			return DEF_WASM_PATH;
		return fName;
	},
	onDataSymbolWASMReady: function (hDecoder) {
		decHandle = hDecoder;
		DecodeImageFile("linear-4.jpg");
	},
	onRuntimeInitialized: function () {
		Module.ccall('InitLib', 'number', ['array'], [str2AsciiArr("")] );
	},
	onAbort: function () {
		console.log("onModuleAbort");
	},
};


// include JS file
eval(fs.readFileSync('datasymbol-sdk.js')+'');


function str2AsciiArr(str) {
	var byteStr = new Uint8Array(str.length + 1);
	for (var i = 0; i < str.length; ++i) {
		var charCode = str.charCodeAt(i);
		if (charCode <= 0xFF)
			byteStr[i] = charCode;
	}
	byteStr[i] = 0x00;
	return byteStr;
}

function SetProperty(decHandle, propName, propVal) {
	return Module.ccall('setProperty',
			'number',
			['number', 'array', 'array'],
			[decHandle, str2AsciiArr(propName), new Uint8Array(new Uint32Array([propVal]).buffer)]);
}


function DecodeFrame(decHandle, pFrameBuf, width, height) {
	//decode and collect barcodes in "barcodeResult"
	var barcodeResult = [];
	var res = _DecodeRGBA(decHandle, pFrameBuf, width, height);
	var bufLen = width * height * 4;
	if (res == 0) {
		var numBarodes = _getResNum(decHandle);

		for (var i = 0; i < numBarodes; ++i) {
			var barcode = {};
			var dataLen = _getResData(decHandle, pFrameBuf, bufLen, i);
			if (dataLen >= 0) {
				//barcode data
				barcode.data = [];
				for (var j = 0; j < dataLen; j++)
					barcode.data.push(Module.HEAPU8[pFrameBuf + j]);

				//collect points
				barcode.points = [];
				res = _getResPoints(decHandle, pFrameBuf, bufLen, i);
				for (var j = 0; j < 4; j++)
					barcode.points.push( { x: Module.getValue(pFrameBuf + j * 8, 'i32'), y: Module.getValue(pFrameBuf + j * 8 + 4, 'i32') } );

				//type
				barcode.bt = _getResBarcodeType(decHandle, i);

				barcodeResult.push(barcode);
			}
		}
	}

	return barcodeResult;
}

function DecodeImageFile(imageSrc) {
	//read image file
	const fileData = fs.readFileSync(imageSrc, {flag:'r'});
	var img = new canvas.Image();
	img.src = fileData;

	//draw image into the canvas and get RGBA data
	var cnv = new canvas.Canvas(img.width, img.height);   
	var ctx = cnv.getContext('2d');
	ctx.drawImage(img, 0, 0, img.width, img.height);
	var imageData = ctx.getImageData(0, 0, img.width, img.height);

	//set any barcode reader properties
	var res = SetProperty(decHandle, 'uiLinearFindBarcodes', 4);

	//allocate frame buffer
	var bufLen = img.width * img.height * 4;
	var pFrameBuf = _malloc(bufLen);

	//decode frame
	for( var i=1; i <= 3; ++i ) {
		console.log(i + ':');

		Module.HEAPU8.set(imageData.data, pFrameBuf);
		var barcodeResult = DecodeFrame(decHandle, pFrameBuf, img.width, img.height);

		//print result
		console.log('Decoded: ' + barcodeResult.length);
		for( var n=0; n < barcodeResult.length; ++n )
			console.log( String.fromCharCode.apply(null, barcodeResult[n].data) );

		console.log('');
	}

	//free resources
	_FreeLib(decHandle);
	if (pFrameBuf)
		_free(pFrameBuf);
}

